/*
 procinfo -- list process information
 Copyright (C) 2007 Daniel Molina Wegener <dmw@coder.cl>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307  US
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <getopt.h>

#include "pinfo.h"
#include "proc_tool.h"


const char pinfo_options[] = "lLp:m:d:H:ha";

static void usage (const char *progname);
static void about (const char *progname);

static int unit_mode = MODE_BYTES;

int
main (int argc, char **argv) {

	int c, errflg = 0, lst_mode = 0, mat_mode = 0, action_mode = 0;
	char *matcher = (char *)NULL;
    extern char *optarg;

	if (argc == 1) {
		usage (argv[0]);
	}

    while ((c = getopt(argc, argv, pinfo_options)) != -1) {
        switch ((int)c) {
		case 'H':
			switch ((int)optarg[0]) {
				case 'B':
				case 'b':
					unit_mode = MODE_BYTES;
					break;
				case 'K':
				case 'k':
					unit_mode = MODE_KBYTES;
					break;
				case 'M':
				case 'm':
					unit_mode = MODE_MBYTES;
					break;
				case 'G':
				case 'g':
					unit_mode = MODE_GBYTES;
					break;
				default:
					unit_mode = MODE_BYTES;
					break;
			}
			break;
        case 'l':
			lst_mode = MODE_SHORT_LIST;
			break;
        case 'L':
			lst_mode = MODE_LONG_LIST;
			break;
        case 'p':
			mat_mode = MODE_PROC_MATCHER;
			matcher = optarg;
            break;
        case 'm':
			mat_mode = MODE_MOD_MATCHER;
			matcher = optarg;
            break;
		case 'd':
			switch ((int)(optarg[0])) {
				case 'k':
					action_mode = ACTION_KILL;
					break;
				case 'u':
					action_mode = ACTION_UNLOAD;
					break;
				case 'l':
					action_mode = ACTION_UNLOCK;
					break;
				default:
					usage (argv[0]);
					break;
			}
			break;
        case 'h':
			usage (argv[0]);
			exit (EXIT_FAILURE);
        case 'a':
			about (argv[0]);
			exit (EXIT_FAILURE);
        case '?':
			fprintf(stderr, "%s: unknown option -%c\n", argv[0], optopt);
            errflg++;
        }
    }

	if (mat_mode == 0) {
		switch (lst_mode) {
			case MODE_SHORT_LIST:
				pinfo_shortlist (unit_mode);
				exit (EXIT_SUCCESS);
			case MODE_LONG_LIST:
				pinfo_longlist (unit_mode);
				exit (EXIT_SUCCESS);
		}
	} else {
		switch (mat_mode) {
			case MODE_PROC_MATCHER:
				pinfo_psearch_list (matcher, lst_mode == MODE_LONG_LIST ? 1 : 0, action_mode, unit_mode);
				break;
			case MODE_MOD_MATCHER:
				pinfo_msearch_list (matcher, lst_mode == MODE_LONG_LIST ? 1 : 0, action_mode, unit_mode);
				exit (EXIT_FAILURE);
		}
	}

    if (errflg > 0) {
        usage (argv[0]);
        exit (EXIT_FAILURE);
    }

	return 0;
}


static void
usage (const char *progname) {
	const char msg[] =
		"%s usage:\n"
		"%s <options>\n"
		"options:\n"
		"	-l			short process listing\n"
		"	-L			long process listing\n"
		"	-p <pcre>		search for processs matching <pcre>\n"
		"	-m <pcre>		search for modules matching <pcre>\n"
		"	-d <action>		do <action> with found processes\n"
		"	-H (g|m|k|b)		displays human redeable image sizes\n"
		"	-h			displays this help message\n"
		"	-a			show the copyleft notice\n"
		"pcre:\n"
		"	Perl Compatible Regular Expression\n"
		"action (only available with -p and -m options):\n"
		"	k			kills matching process\n"
		"	u			unload matching module\n"
		"	l			unlock matching module/process\n";
	if (progname != (const char *)NULL) {
		fprintf (stderr, msg, progname, progname);
	}
}


static void
about (const char *progname) {
	const char msg[] =
		"\n"
		"%s - Process Information and Tools\n\n"
		"Copyright (c) 2007 Daniel Molina Wegener <dmw@coder.cl>\n\n"
	    "This program is free software; you can redistribute it and/or\n"
		"modify it under the terms of the GNU General Public License as\n"
		"published by the Free Software Foundation; either version 2 of the\n"
		"License, or (at your option) any later version.\n\n"
		"This program is distributed in the hope that it will be useful, but\n"
		"WITHOUT ANY WARRANTY; without even the implied warranty of\n"
		"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\n"
		"General Public License for more details.\n\n"
		"You should have received a copy of the GNU General1 Public License\n"
		"along with this program; if not, write to the Free Software\n"
		"Foundation, Inc., 59 Temple Place - Suite 330, Boston,\n"
		"MA 02111-1307, USA.\n"
		"\n";

	if (progname != (const char *)NULL) {
		fprintf (stderr, msg, progname);
	}
}


