/*
 procinfo -- list process information
 Copyright (C) 2007 Daniel Molina Wegener <dmw@coder.cl>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307  US
 */

#define PCRE_STATIC 1

#include <stdlib.h>
#include <stdio.h>
#include <windows.h>
#include <psapi.h>

#include "proc_tool.h"


int
pinfo_paction (HANDLE pid, DWORD proc_current, const char *path, int mode) {
	int r = 0;
	if (pid > 0 && mode != 0 && mode < 100) {
		switch (mode) {
			case ACTION_KILL:
				fprintf (stderr, "[%ld] Killing process... ", proc_current);
				r = pinfo_pkill (pid, proc_current);
				if (r != 0) {
					fprintf (stderr, "done!\n");
				} else {
					fprintf (stderr, "failed!\n");
				}
				break;
			case ACTION_UNLOCK:
				fprintf (stderr, "[%ld] Unlocking process executable... ", proc_current);
				r = pinfo_punlock (pid, proc_current, path);
				if (r != 0) {
					fprintf (stderr, "done!\n");
				} else {
					fprintf (stderr, "failed!\n");
				}
			default:
				fprintf (stderr, "[%ld] Unknown action %d\n", proc_current, mode);
				break;
		}
	}
	return r;
}


int
pinfo_pkill (HANDLE pid, DWORD proc_current) {
	DWORD exit_code = 0;
	int r = 0;
	if (pid > 0) {
		if ((GetExitCodeProcess (pid, &exit_code)) == TRUE) {
			if ((TerminateProcess (pid, exit_code)) == TRUE) {
				r = 1;
			}
		}
	}
	return r;
}


int
pinfo_punlock (HANDLE pid, DWORD proc_current, const char *path) {
	int r = 0;
	HFILE fh = 0;
	OFSTRUCT fs;
	if (pid > 0 && proc_current > 0 && path != (const char *)NULL) {
		fh = OpenFile (path, (LPOFSTRUCT)&fs, OF_READ | OF_PARSE);
		if (fh > 0) {
			UnlockFile ((HANDLE)fh, 0, fs.cBytes, 0, fs.cBytes);
			UnlockFile ((HANDLE)pid, 0, fs.cBytes, 0, fs.cBytes);
			_lclose (fh);
			r = 1;
		}
	}
	return r;
}


int
pinfo_maction (HANDLE pid, DWORD proc_current, HMODULE module, char *mname, int mode) {
	int r = 0;
	if (pid > 0 && mode != 0 && module > 0 && mode > 99) {
		switch (mode) {
			case ACTION_UNLOAD:
				fprintf (stderr, "[%ld] Unloading module '%s'... ", proc_current, mname);
				r = pinfo_munload (pid, proc_current, module);
				if (r != 0) {
					fprintf (stderr, "done!\n");
				} else {
					fprintf (stderr, "failed!\n");
				}
				break;
			case ACTION_UNLOCK:
				fprintf (stderr, "[%ld] Unlocking process module '%s'... ", proc_current, mname);
				r = pinfo_munlock (pid, proc_current, module, mname);
				if (r != 0) {
					fprintf (stderr, "done!\n");
				} else {
					fprintf (stderr, "failed!\n");
				}
				break;
			default:
				fprintf (stderr, "[%ld] Unknown action %d\n", proc_current, mode);
				break;
		}
	}
	return r;
}


int
pinfo_munload (HANDLE pid, DWORD proc_current, HMODULE module) {
	int r = 0;
	if (pid > 0 && proc_current > 0 && module > 0) {
		if ((FreeLibrary (module)) == TRUE) {
			r = 1;
		}
	}
	return r;
}


int
pinfo_munlock (HANDLE pid, DWORD proc_current, HMODULE module, const char *path) {
	int r = 0;
	HFILE fh = 0;
	OFSTRUCT fs;
	if (pid > 0 && proc_current > 0 && path != (const char *)NULL) {
		fh = OpenFile (path, (LPOFSTRUCT)&fs, GENERIC_READ | OF_READ | OF_PARSE);
		if (fh > 0) {
			UnlockFile ((HANDLE)fh, 0, fs.cBytes, 0, fs.cBytes);
			UnlockFile ((HANDLE)pid, 0, fs.cBytes, 0, fs.cBytes);
			_lclose (fh);
			r = 1;
		}
	}
	return r;
}


double
unit_conversion (DWORD unit, int mode) {
	switch (mode) {
		case MODE_BYTES:
			return (double)unit;
			break;
		case MODE_KBYTES:
			return ((double)unit / 1024.0);
			break;
		case MODE_MBYTES:
			return (((double)unit / 1024.0) / 1024.0);
			break;
		case MODE_GBYTES:
			return ((((double)unit / 1024.0) / 1024.0) / 1024.0);
			break;
		default:
			return (double)unit;
	}
}


void
print_unit (DWORD unit, int mode, int indent) {
	const char msg[] = "%sImage Size  : %.2f %s\n";
	char *unitstr;
	switch (mode) {
		case MODE_BYTES:
			unitstr = "bytes";
			break;
		case MODE_KBYTES:
			unitstr = "Kb";
			break;
		case MODE_MBYTES:
			unitstr = "Mb";
			break;
		case MODE_GBYTES:
			unitstr = "Gb";
			break;
		default:
			unitstr = "bytes";
	}
	printf (msg, indent == 1 ? "\t" : "\t\t", unit_conversion(unit, mode), unitstr);
}

